#include "BPopUpButton.h"
#include "BInfoSettingsWin.h"
#include "BInfoSettingsFile.h"
#include "StringView.h"

#include "tools.h"
#include "const.h"

#include "BBorder.h"

BInfoSettingsWin :: BInfoSettingsWin(BRect rect)
	: BWindow(rect, "BInfo Settings", B_TITLED_WINDOW, B_NOT_CLOSABLE|B_NOT_RESIZABLE|B_NOT_ZOOMABLE|B_NOT_MINIMIZABLE)
{
	load();

	BView *mainview;
	
	Lock();
	SetDiscipline(FALSE);

	// CREATION DE LA BVIEW PRINCIPALE
	rect.Set(0, 0, Frame().IntegerWidth(), Frame().IntegerHeight());
	mainview = new BView(rect, "", B_FOLLOW_ALL, B_WILL_DRAW);
	AddChild(mainview);
	mainview->SetViewColor(220,220,220);


	// CREATION FRAME DISPLAY
	rect.Set(OFFSET, OFFSET, mainview->Frame().IntegerWidth() - OFFSET, mainview->Frame().IntegerHeight() - OFFSET);
	BBorder *border=new BBorder(mainview, rect, "", B_FOLLOW_LEFT_RIGHT, " Display ");


	// CREATION DES CHECKBOX
	int checkwidth = border->innerView->Frame().IntegerWidth();
	int checkleft = OFFSET;
	int checktop = OFFSET;

	rect.Set(checkleft, checktop, checkwidth, 0);
	checkMachineInfo = createCheckBox(border->innerView, rect, "Machine Info", NULL);

	checktop=checkMachineInfo->Frame().bottom + 2*OFFSET;
	rect.Set(checkleft, checktop, checkwidth, 0);
	checkScreenInfo = createCheckBox(border->innerView, rect, "Screen Info", NULL);

	checktop=checkScreenInfo->Frame().bottom + 2*OFFSET;
	rect.Set(checkleft, checktop, checkwidth, 0);
	checkSystemInfo = createCheckBox(border->innerView, rect, "System Info", 'S');

	checkleft += 4*OFFSET;
	checktop=checkSystemInfo->Frame().bottom + OFFSET;
	rect.Set(checkleft, checktop, checkwidth, 0);
	checkMemory = createCheckBox(border->innerView, rect, "Memory", NULL);

	checktop=checkMemory->Frame().bottom + OFFSET;
	rect.Set(checkleft, checktop, checkwidth, 0);
	checkSems = createCheckBox(border->innerView, rect, "Semaphores", NULL);

	checktop=checkSems->Frame().bottom + OFFSET;
	rect.Set(checkleft, checktop, checkwidth, 0);
	checkPorts = createCheckBox(border->innerView, rect, "Ports", NULL);

	checktop=checkPorts->Frame().bottom + OFFSET;
	rect.Set(checkleft, checktop, checkwidth, 0);
	checkThreads = createCheckBox(border->innerView, rect, "Threads", NULL);

	checktop=checkThreads->Frame().bottom + OFFSET;
	rect.Set(checkleft, checktop, checkwidth, 0);
	checkTeams = createCheckBox(border->innerView, rect, "Teams", NULL);

	checkleft -= 4*OFFSET;
	checktop=checkTeams->Frame().bottom + 2*OFFSET;
	rect.Set(checkleft, checktop, checkwidth, 0);
	checkDiskInfo = createCheckBox(border->innerView, rect, "Disk Info", NULL);
	
	border->ResizeTo(border->Frame().IntegerWidth(), 
			OFFSET + checkDiskInfo->Frame().bottom+border->Frame().IntegerHeight()-border->innerView->Frame().IntegerHeight());


	// CREATION FRAME REFRESH RATE
	rect.Set(OFFSET, OFFSET + border->Frame().bottom, mainview->Frame().IntegerWidth() - OFFSET, border->Frame().bottom + 50);
	BBorder *border2=new BBorder(mainview, rect, "", B_FOLLOW_LEFT_RIGHT, " Refresh rate ");

	// CREATION POPUPBUTTON
	StringView *str = new StringView(border2->innerView, BPoint(OFFSET,OFFSET), "Every  ");
	rect.Set(str->Frame().right,OFFSET-2,border2->innerView->Frame().right,50);
	
	char *labels[] = {"0.5 second", "1 second", "3 seconds", "5 seconds", "10 seconds", 0};
	refreshButton = new BPopUpButton(border2->innerView, rect, "Refresh rate", labels);
	
	border2->ResizeTo(border2->Frame().IntegerWidth(),
		OFFSET + str->Frame().bottom+border2->Frame().IntegerHeight()-border2->innerView->Frame().IntegerHeight());
	
	
	// CREATION DES BOUTONS
	int buttonwidth = (mainview->Frame().IntegerWidth() - 4 * OFFSET) / 3;
	int buttonleft = OFFSET;
	int buttontop = border2->Frame().bottom + 10;

	rect.Set(buttonleft, buttontop, buttonleft + buttonwidth, 0);
	createButton(mainview, rect, "Use", 'U');
	
	buttonleft += OFFSET + buttonwidth;
	rect.Set(buttonleft, buttontop, buttonleft + buttonwidth, 0);
	createButton(mainview, rect, "Save", 'S');
	
	buttonleft += OFFSET + buttonwidth;
	rect.Set(buttonleft, buttontop, buttonleft + buttonwidth, 0);
	BButton *button = createButton(mainview, rect, "Cancel", 'C');

	ResizeTo(Frame().IntegerWidth(),button->Frame().bottom + OFFSET);
	
	
	initSettings();
		
	Unlock();
}


BButton *BInfoSettingsWin :: createButton(BView *parent, BRect rect, const char* label, ulong type)
{
	BMessage *message = new BMessage(MCUSTOM_BUTTON);
	
	message->AddLong("type", type);
	
	BButton *button = new BButton(rect,
								  "",
								  label,
								  message,
								  NULL,
								  B_WILL_DRAW);
	parent->AddChild(button);

	return button;
}

BCheckBox *BInfoSettingsWin :: createCheckBox(BView *parent, BRect rect, const char* label, ulong type)
{
	BMessage *message;
	
	if(type != NULL)
	{
		message = new BMessage(MCUSTOM_CHECKBOX);
		message->AddLong("type", type);
	}
	else
		message = NULL;

	BCheckBox *button = new BCheckBox(rect,
								  "",
								  label,
								  message,
								  NULL,
								  B_WILL_DRAW);
	parent->AddChild(button);

	return button;
}


void BInfoSettingsWin :: MessageReceived(BMessage *message)
{
	char type;

	switch(message->what) 
	{
		case MCUSTOM_CHECKBOX:
			type = (char)(message->FindLong("type"));
			switch(type)
			{
				case 'S':
					bool value = (checkSystemInfo->Value()==B_CONTROL_ON);
					
					Lock();
					checkMemory->SetEnabled(value);
					checkSems->SetEnabled(value);
					checkPorts->SetEnabled(value);
					checkThreads->SetEnabled(value);
					checkTeams->SetEnabled(value);
					Unlock();
					
					break;	
			}
			break;
			
		case MCUSTOM_BUTTON:
			type = (char)(message->FindLong("type"));
			switch(type)
			{
				case 'S':
				case 'U':
					if(settingsOK())
					{
						if(settingsChanged())					
							be_app->PostMessage(MCUSTOM_SETTINGSCHANGED);

						if(type=='S')
							save();

						Hide();
					}
					break;
					
				case 'C':
					Hide();
					initSettings();
					break;
			}			
			break;
			
		default:
			inherited::MessageReceived(message);
			break;	
	}
}

void BInfoSettingsWin :: load()
{
	BInfoSettingsFile *loader = new BInfoSettingsFile();
	long value;
	
	refreshRate = DEF_REFRESHRATE;
	displayMachineInfo = B_CONTROL_ON;
	displayScreenInfo = B_CONTROL_ON;
	displaySystemInfo = B_CONTROL_ON;
	displayDiskInfo = B_CONTROL_ON;
	displayMemory = B_CONTROL_ON;
	displaySems = B_CONTROL_ON;
	displayPorts = B_CONTROL_ON;
	displayThreads = B_CONTROL_ON;
	displayTeams = B_CONTROL_ON;

	if(loader->OpenRead() == B_NO_ERROR)
	{
		if(loader->Read(SAVE_MACHINE, &value) == B_NO_ERROR)
			displayMachineInfo = value;

		if(loader->Read(SAVE_SCREEN, &value) == B_NO_ERROR)
			displayScreenInfo = value;
			
		if(loader->Read(SAVE_SYSTEM, &value) == B_NO_ERROR)
			displaySystemInfo = value;
			
		if(loader->Read(SAVE_DISK, &value) == B_NO_ERROR)
			displayDiskInfo = value;
			
		if(loader->Read(SAVE_MEM, &value) == B_NO_ERROR)
			displayMemory = value;
			
		if(loader->Read(SAVE_SEM, &value) == B_NO_ERROR)
			displaySems = value;
			
		if(loader->Read(SAVE_PORT, &value) == B_NO_ERROR)
			displayPorts = value;
			
		if(loader->Read(SAVE_THRD, &value) == B_NO_ERROR)
			displayThreads = value;
		
		if(loader->Read(SAVE_TEAM, &value) == B_NO_ERROR)
			displayTeams = value;

		if(loader->Read(SAVE_RFRSHRATE, &value) == B_NO_ERROR)
			refreshRate = value;
			
		loader->Close();
	}
	
	delete loader;
}

void BInfoSettingsWin :: save()
{
	BInfoSettingsFile *saver = new BInfoSettingsFile();
	if(saver->OpenSave() == B_NO_ERROR)
	{
		saver->Write(SAVE_MACHINE, displayMachineInfo);
		saver->Write(SAVE_SCREEN, displayScreenInfo);
		saver->Write(SAVE_SYSTEM, displaySystemInfo);
		saver->Write(SAVE_DISK, displayDiskInfo);
		saver->Write(SAVE_MEM, displayMemory);
		saver->Write(SAVE_SEM, displaySems);
		saver->Write(SAVE_PORT, displayPorts);
		saver->Write(SAVE_THRD, displayThreads);
		saver->Write(SAVE_TEAM, displayTeams);
		saver->Write(SAVE_RFRSHRATE, refreshRate);
		saver->Close();
	}
	
	delete saver;
}

bool BInfoSettingsWin :: settingsOK()
{
	bool res=TRUE;

	if((checkMemory->Value() == B_CONTROL_OFF)
		&& (checkSems->Value() == B_CONTROL_OFF)
		&& (checkPorts->Value() == B_CONTROL_OFF)
		&& (checkThreads->Value() == B_CONTROL_OFF)
		&& (checkTeams->Value() == B_CONTROL_OFF))
	{
		Lock();
		checkSystemInfo->SetValue(B_CONTROL_OFF);
		Unlock();
		
		BMessage *message = new BMessage(MCUSTOM_CHECKBOX);
		message->AddLong("type", 'S');
		PostMessage(message);
	}
	
	if((checkMachineInfo->Value() == B_CONTROL_OFF)
		&& (checkScreenInfo->Value() == B_CONTROL_OFF)
		&& (checkSystemInfo->Value() == B_CONTROL_OFF)
		&& (checkDiskInfo->Value() == B_CONTROL_OFF))
	{
		MsgBox("You must display at least one panel.");
		res=FALSE;
	}
	
	return res;
}

bool BInfoSettingsWin :: settingsChanged()
{
	bool res=FALSE;
	
	Lock();
	
	if(checkMachineInfo->Value()!=displayMachineInfo)
	{
		displayMachineInfo=checkMachineInfo->Value();
		res=TRUE;
	}

	if(checkScreenInfo->Value()!=displayScreenInfo)
	{
		displayScreenInfo=checkScreenInfo->Value();
		res=TRUE;
	}

	if(checkSystemInfo->Value()!=displaySystemInfo)
	{
		displaySystemInfo=checkSystemInfo->Value();
		res=TRUE;
	}

	if(checkDiskInfo->Value()!=displayDiskInfo)
	{
		displayDiskInfo=checkDiskInfo->Value();
		res=TRUE;
	}

	if(checkMemory->Value()!=displayMemory)
	{
		displayMemory=checkMemory->Value();
		res=TRUE;
	}

	if(checkSems->Value()!=displaySems)
	{
		displaySems=checkSems->Value();
		res=TRUE;
	}

	if(checkPorts->Value()!=displayPorts)
	{
		displayPorts=checkPorts->Value();
		res=TRUE;
	}

	if(checkThreads->Value()!=displayThreads)
	{
		displayThreads=checkThreads->Value();
		res=TRUE;
	}

	if(checkTeams->Value()!=displayTeams)
	{
		displayTeams=checkTeams->Value();
		res=TRUE;
	}
	
	int i=refreshButton->Selected();
	if(i!=refreshRate)
	{
		refreshRate=i;
		res=TRUE;
	}

	Unlock();	
	
	return res;
}

void BInfoSettingsWin :: ShowMe()
{
	SetWorkspaces(B_CURRENT_WORKSPACE);
	if(IsHidden())
	{
		screen_info scrinf;
		get_screen_info(&scrinf);

		if(    Frame().left < 0
			|| Frame().right > scrinf.frame.IntegerWidth()
			|| Frame().top < 0
			|| Frame().bottom > scrinf.frame.IntegerHeight())
			MoveTo(DEF_SETTWIN_X, DEF_SETTWIN_Y);
		
		Show();
	}
	else
		Activate();
}

void BInfoSettingsWin :: initSettings()
{
	Lock();
	checkMachineInfo->SetValue(displayMachineInfo);
	checkScreenInfo->SetValue(displayScreenInfo);
	checkSystemInfo->SetValue(displaySystemInfo);
	checkMemory->SetValue(displayMemory);
	checkSems->SetValue(displaySems);
	checkPorts->SetValue(displayPorts);
	checkThreads->SetValue(displayThreads);
	checkTeams->SetValue(displayTeams);
	checkDiskInfo->SetValue(displayDiskInfo);
	
	refreshButton->SetCurrent(refreshRate);
	Unlock();
	
	BMessage *message = new BMessage(MCUSTOM_CHECKBOX);
	message->AddLong("type", 'S');
	PostMessage(message);
	
}

long BInfoSettingsWin :: RefreshRate()
{
	long ret;
	
	switch(refreshRate)
	{
		case VREFRESH_05: ret = REFRESH_05; break;
		case VREFRESH_1:  ret = REFRESH_1;  break;
		case VREFRESH_3:  ret = REFRESH_3;  break;
		case VREFRESH_5:  ret = REFRESH_5;  break;
		case VREFRESH_10: ret = REFRESH_10; break;
	}

	return ret;
}
