#include "BInfoApp.h"
#include "BInfoView.h"
#include "BInfoWin.h"
#include "DiskView.h"

#include "const.h"
#include "tools.h"

BInfoView :: BInfoView(BWindow *parent, BRect frame) :
		BView(frame, "", B_FOLLOW_LEFT_RIGHT, B_WILL_DRAW|B_PULSE_NEEDED)
{
	first=TRUE;
	parent->AddChild(this);
}

void BInfoView :: Init()
{
	BInfoSettingsWin *settings=((BInfoApp *)be_app)->Settings();
	BRect winrect;
	long top = 0;

	SetViewColor(220,220,220);

	// CREATION DU FRAME "Machine Info"
	if(settings->DisplayMachineInfo())
	{
		winrect.Set(OFFSET, OFFSET, Frame().IntegerWidth() - OFFSET, 0);
		borderMachineInfo = new BBorder(this, winrect, "", B_FOLLOW_LEFT_RIGHT, " Machine info ");
		
		DisplayMachineInfo();
		top = borderMachineInfo->Frame().bottom;
	}

	// CREATION DU FRAME "Screen Info"
	if(settings->DisplayScreenInfo())
	{
		winrect.Set(OFFSET, OFFSET + top, Frame().IntegerWidth() - OFFSET, 0);
		borderScreenInfo = new BBorder(this, winrect, "", B_FOLLOW_LEFT_RIGHT, " Screen info ");
		
		DisplayStaticScreenInfo();
		DisplayDynamicScreenInfo();
		top = borderScreenInfo->Frame().bottom;
	}
	
	// CREATION DU FRAME "System Info"
	if(settings->DisplaySystemInfo())
	{	
		winrect.Set(OFFSET, OFFSET + top, Frame().IntegerWidth() - OFFSET, 0);
		borderSystemInfo = new BBorder(this, winrect, "", B_FOLLOW_LEFT_RIGHT, " System info ");
			
		DisplayStaticSystemInfo();
		top = borderSystemInfo->Frame().bottom;
	}
	
	// CREATION DU FRAME "Disk Info"
	if(settings->DisplayDiskInfo())
	{
		winrect.Set(OFFSET, OFFSET + top, Frame().IntegerWidth() - OFFSET, 0);
		borderDiskInfo = new BBorder(this, winrect, "", B_FOLLOW_LEFT_RIGHT, " Disk info ");
	}
		
	Pulse();
}

void BInfoView :: Pulse()
{
	BInfoSettingsWin *settings=((BInfoApp *)be_app)->Settings();
	BInfoApp *binfoapp = (BInfoApp *)be_app;
	BInfoWin *binfowin = (BInfoWin *)Window();

	binfowin->Lock();
	settings->Lock();
	
	if(binfoapp->newDiskInfo)
	{
		if(settings->DisplayDiskInfo())
			DisplayStaticDiskInfo();
			
		binfowin->ZoomMe();
		binfoapp->newDiskInfo=FALSE;
		binfowin->SetPulseRate(settings->RefreshRate());
	}

	if(settings->DisplaySystemInfo())
		DisplayDynamicSystemInfo();
		
	if(settings->DisplayDiskInfo())
		DisplayDynamicDiskInfo();
	
	settings->Unlock();
	binfowin->Unlock();
}

void BInfoView :: DisplayMachineInfo()
{
	BRect rect;
	char texte[256];
	system_info syst_inf;
	
	get_system_info(&syst_inf);


	// AFFICHAGE DES INFOS MACHINE
	rect=borderMachineInfo->innerView->Frame();
	rect.OffsetTo(B_ORIGIN);
	machinetextes = new LabeledText(borderMachineInfo->innerView, rect, "", B_FOLLOW_LEFT_RIGHT, 85);
	machinetextes->SetFontName("Erich");
	machinetextes->SetFontSize(10);
			
	machinetextes->AddLabel("Machine ID :");
//	machinetextes->AddLabel("Boot Time :");
	machinetextes->AddLabel("CPU Count :");
	machinetextes->AddLabel("CPU Type :");
	machinetextes->AddLabel("CPU Revision :");
	machinetextes->AddLabel("CPU Clock Speed :");
	machinetextes->AddLabel("BUS Clock Speed :");
	machinetextes->CreateLabels();

	
	//Machine ID
	sprintf(texte, "#%X%X", syst_inf.id[0], syst_inf.id[1]);
	machinetextes->SetText(0, texte);
	
	//Boot Time
//	sprintf(texte, "%.1f ?", syst_inf.boot_time/*/(60*60*24*7*52)*/);
//	machinetextes->SetText(1, texte);
	
	//CPU Count
	sprintf(texte, "%ld", syst_inf.cpu_count);
	machinetextes->SetText(1, texte);
	
	//CPU Type
	switch(syst_inf.cpu_type)
	{
		case B_CPU_PPC_601 :  strcpy(texte, "Motorola PowerPC 601"); break;
		case B_CPU_PPC_603 :  strcpy(texte, "Motorola PowerPC 603"); break;
		case B_CPU_PPC_603e : strcpy(texte, "Motorola PowerPC 603e"); break;
		case B_CPU_PPC_604 :  strcpy(texte, "Motorola PowerPC 604"); break;
		case B_CPU_PPC_604e : strcpy(texte, "Motorola PowerPC 604e"); break;
		case B_CPU_PPC_686 :  strcpy(texte, "Motorola PowerPC 686"); break;
		default :             strcpy(texte, "Unknown"); break;
	}
	machinetextes->SetText(2, texte);

	//CPU Revision
	sprintf(texte, "%ld", syst_inf.cpu_revision);
	machinetextes->SetText(3, texte);

	//CPU Clock Speed
	sprintf(texte, "%ld MHz", arrond(syst_inf.cpu_clock_speed/1e6));
	machinetextes->SetText(4, texte);

	//BUS Clock Speed
	sprintf(texte, "%ld MHz", arrond(syst_inf.bus_clock_speed/1e6));
	machinetextes->SetText(5, texte);
	
	borderMachineInfo->ResizeTo(borderMachineInfo->Frame().IntegerWidth(),
		machinetextes->Frame().bottom+borderMachineInfo->Frame().IntegerHeight()-borderMachineInfo->innerView->Frame().IntegerHeight());
	ResizeTo(Frame().IntegerWidth(), borderMachineInfo->Frame().bottom + OFFSET);
}

void BInfoView :: DisplayStaticScreenInfo()
{
	// AFFICHAGE DES INFOS MACHINE
	BRect rect=borderScreenInfo->innerView->Frame();
	rect.OffsetTo(B_ORIGIN);
	screentextes = new LabeledText(borderScreenInfo->innerView, rect, "", B_FOLLOW_LEFT_RIGHT, 64);
	screentextes->SetFontName("Erich");
	screentextes->SetFontSize(10);
			
	screentextes->AddLabel("Resolution :");
	screentextes->AddLabel("Color space :");
	screentextes->CreateLabels();

	borderScreenInfo->ResizeTo(borderScreenInfo->Frame().IntegerWidth(),
		screentextes->Frame().bottom+borderScreenInfo->Frame().IntegerHeight()-borderScreenInfo->innerView->Frame().IntegerHeight());
	ResizeTo(Frame().IntegerWidth(), borderScreenInfo->Frame().bottom + OFFSET);
}

void BInfoView :: DisplayDynamicScreenInfo()
{
	BInfoSettingsWin *settings=((BInfoApp *)be_app)->Settings();
	char texte[256];
	screen_info scrinf;

	if(settings->DisplayScreenInfo())
	{
		get_screen_info(&scrinf);
		
		// RESOLUTION
		sprintf(texte, "%d x %d", scrinf.frame.IntegerWidth()+1, scrinf.frame.IntegerHeight()+1);
		screentextes->SetText(0, texte);
	
		// COLOR SPACE
		switch(scrinf.mode)
		{
			case B_MONOCHROME_1_BIT: strcpy(texte, "Monochrome 1 bit"); break;
			case B_GRAYSCALE_8_BIT: strcpy(texte, "Grayscale 8 bits"); break;
			case B_COLOR_8_BIT: strcpy(texte, "Color 8 bits"); break;
			case B_RGB_16_BIT: strcpy(texte, "RGB 16 bits"); break;
			case B_RGB_32_BIT: strcpy(texte, "RGB 32 bits"); break;
			default : strcpy(texte, "Unknown"); break;
		}
		screentextes->SetText(1, texte);
	}
}

void BInfoView :: DisplayStaticSystemInfo()
{
	BInfoSettingsWin *settings=((BInfoApp *)be_app)->Settings();
	BRect rect;
	long top = 0;
	
	system_info syst_inf;
	get_system_info(&syst_inf);

	
	// AFFICHAGE DE LA MEMOIRE
	if(settings->DisplayMemory())
	{
		rect=borderSystemInfo->innerView->Frame();
		rect.OffsetTo(B_ORIGIN);
		systemtextes1 = new LabeledText(borderSystemInfo->innerView, rect, "", B_FOLLOW_LEFT_RIGHT, 100);
		systemtextes1->SetFontName("Erich");
		systemtextes1->SetFontSize(10);
	
		systemtextes1->AddLabel("Used memory pages :");
		systemtextes1->AddLabel("Page faults :");
		systemtextes1->AddLabel("Used memory :");
		systemtextes1->CreateLabels();
		
		// GAUGE D'AFFICHAGE DE LA MEMOIRE
		rect.Set(0, 0, borderSystemInfo->innerView->Frame().IntegerWidth(), 15);
		rect.OffsetTo(0, systemtextes1->Frame().bottom);		
		bgaugemem = new BGauge(rect, "", B_FOLLOW_LEFT_RIGHT, syst_inf.max_pages);
		borderSystemInfo->innerView->AddChild(bgaugemem);
		
		top = bgaugemem->Frame().bottom + 10;
	}
	

	// AFFICHAGE DES SEMAPHORES
	if(settings->DisplaySems())
	{
		rect=borderSystemInfo->innerView->Frame();
		rect.OffsetTo(0, top);
		systemtextes2 = new LabeledText(borderSystemInfo->innerView, rect, "", B_FOLLOW_LEFT_RIGHT, 100);
		systemtextes2->SetFontName("Erich");
		systemtextes2->SetFontSize(10);
	
		systemtextes2->AddLabel("Used semaphores :");
		systemtextes2->CreateLabels();
	
		// GAUGE D'AFFICHAGE DES SEMAPHORES
		rect.Set(0, 0, borderSystemInfo->innerView->Frame().IntegerWidth(), 15);
		rect.OffsetTo(0, systemtextes2->Frame().bottom);		
		bgaugesem = new BGauge(rect, "", B_FOLLOW_LEFT_RIGHT, syst_inf.max_sems);
		borderSystemInfo->innerView->AddChild(bgaugesem);
	
		top = bgaugesem->Frame().bottom + 10;
	}
	
	
	// AFFICHAGE DES PORTS
	if(settings->DisplayPorts())
	{
		rect=borderSystemInfo->innerView->Frame();
		rect.OffsetTo(0, top);
		systemtextes3 = new LabeledText(borderSystemInfo->innerView, rect, "", B_FOLLOW_LEFT_RIGHT, 100);
		systemtextes3->SetFontName("Erich");
		systemtextes3->SetFontSize(10);
	
		systemtextes3->AddLabel("Used ports :");
		systemtextes3->CreateLabels();
	
		// GAUGE D'AFFICHAGE DES PORTS
		rect.Set(0, 0, borderSystemInfo->innerView->Frame().IntegerWidth(), 15);
		rect.OffsetTo(0, systemtextes3->Frame().bottom);		
		bgaugeport = new BGauge(rect, "", B_FOLLOW_LEFT_RIGHT, syst_inf.max_ports);
		borderSystemInfo->innerView->AddChild(bgaugeport);
		
		top = bgaugeport->Frame().bottom + 10;
	}
	
	
	// AFFICHAGE DES THREADS
	if(settings->DisplayThreads())
	{
		rect=borderSystemInfo->innerView->Frame();
		rect.OffsetTo(0, top);
		systemtextes4 = new LabeledText(borderSystemInfo->innerView, rect, "", B_FOLLOW_LEFT_RIGHT, 100);
		systemtextes4->SetFontName("Erich");
		systemtextes4->SetFontSize(10);
	
		systemtextes4->AddLabel("Used threads :");
		systemtextes4->CreateLabels();
	
		// GAUGE D'AFFICHAGE DES THREADS
		rect.Set(0, 0, borderSystemInfo->innerView->Frame().IntegerWidth(), 15);
		rect.OffsetTo(0, systemtextes4->Frame().bottom);		
		bgaugethread = new BGauge(rect, "", B_FOLLOW_LEFT_RIGHT, syst_inf.max_threads);
		borderSystemInfo->innerView->AddChild(bgaugethread);
		
		top = bgaugethread->Frame().bottom + 10;
	}
	
	
	// AFFICHAGE DES TEAMS
	if(settings->DisplayTeams())
	{
		rect=borderSystemInfo->innerView->Frame();
		rect.OffsetTo(0, top);
		systemtextes5 = new LabeledText(borderSystemInfo->innerView, rect, "", B_FOLLOW_LEFT_RIGHT, 100);
		systemtextes5->SetFontName("Erich");
		systemtextes5->SetFontSize(10);
	
		systemtextes5->AddLabel("Used teams :");
		systemtextes5->CreateLabels();
	
		// GAUGE D'AFFICHAGE DES TEAMS
		rect.Set(0, 0, borderSystemInfo->innerView->Frame().IntegerWidth(), 15);
		rect.OffsetTo(0, systemtextes5->Frame().bottom);		
		bgaugeteam = new BGauge(rect, "", B_FOLLOW_LEFT_RIGHT, syst_inf.max_teams);
		borderSystemInfo->innerView->AddChild(bgaugeteam);
		
		top = bgaugeteam->Frame().bottom + 10;
	}
	
	
	borderSystemInfo->ResizeTo(borderSystemInfo->Frame().IntegerWidth(),
		top-10+borderSystemInfo->Frame().IntegerHeight()-borderSystemInfo->innerView->Frame().IntegerHeight());
	ResizeTo(Frame().IntegerWidth(), borderSystemInfo->Frame().bottom + OFFSET);
}

void BInfoView :: DisplayDynamicSystemInfo()
{
	BInfoSettingsWin *settings=((BInfoApp *)be_app)->Settings();
	char str[256], mem_used[20], mem_max[20];
	system_info syst_inf;
	
	get_system_info(&syst_inf);
	
	// AFFICHAGE DE LA MEMOIRE
	if(settings->DisplayMemory() && (first || syst_inf.used_pages!=old_syst_inf.used_pages))
	{
		sprintMem(mem_used, syst_inf.used_pages * B_PAGE_SIZE);
		sprintMem(mem_max, syst_inf.max_pages * B_PAGE_SIZE);
		sprintf(str,"%s / %s", mem_used, mem_max);
		systemtextes1->SetText(2, str);
		sprintf(str,"%ld / %ld", syst_inf.used_pages, syst_inf.max_pages);
		systemtextes1->SetText(0, str);
		bgaugemem->SetCurrentValue(syst_inf.used_pages);
	}
	
	if(settings->DisplayMemory() && (first || syst_inf.page_faults!=old_syst_inf.page_faults))
	{
		sprintf(str,"%ld", syst_inf.page_faults);
		systemtextes1->SetText(1, str);
	}
	
	// AFFICHAGE DES SEMAPHORES
	if(settings->DisplaySems() && (first || syst_inf.used_sems!=old_syst_inf.used_sems))
	{
		sprintf(str,"%ld / %ld", syst_inf.used_sems, syst_inf.max_sems);
		systemtextes2->SetText(0, str);
		bgaugesem->SetCurrentValue(syst_inf.used_sems);
	}
	
	// AFFICHAGE DES PORTS
	if(settings->DisplayPorts() && (first || syst_inf.used_ports!=old_syst_inf.used_ports))
	{
		sprintf(str,"%ld / %ld", syst_inf.used_ports, syst_inf.max_ports);
		systemtextes3->SetText(0, str);
		bgaugeport->SetCurrentValue(syst_inf.used_ports);
	}
	
	// AFFICHAGE DES THREADS
	if(settings->DisplayThreads() && (first || syst_inf.used_threads!=old_syst_inf.used_threads))
	{
		sprintf(str,"%ld / %ld", syst_inf.used_threads, syst_inf.max_threads);
		systemtextes4->SetText(0, str);
		bgaugethread->SetCurrentValue(syst_inf.used_threads);
	}
	
	// AFFICHAGE DES TEAMS
	if(settings->DisplayTeams() && (first || syst_inf.used_teams!=old_syst_inf.used_teams))
	{
		sprintf(str,"%ld / %ld", syst_inf.used_teams, syst_inf.max_teams);
		systemtextes5->SetText(0, str);
		bgaugeteam->SetCurrentValue(syst_inf.used_teams);
	}
	
	first = FALSE;
	old_syst_inf = syst_inf;
}

void BInfoView :: DisplayStaticDiskInfo()
{
	BRect rect;
	BVolume this_vol;         
	long counter = 0;
	DiskView *last = NULL, *dv;
		
	while(!diskviews.IsEmpty())
	{
		dv=(DiskView *)diskviews.RemoveItem((long)0);
		dv->RemoveSelf();
		delete dv;
	}

	while(this_vol = volume_at(counter), TRUE)
	{
		if (this_vol.Error() != B_NO_ERROR) 
			break;
		rect=borderDiskInfo->innerView->Frame();
		if(!last)
		{
			rect.OffsetTo(0, 0);
		}
		else
		{
			rect.OffsetTo(0, last->Frame().bottom+10);
		}
		DiskView *dv = new DiskView(borderDiskInfo->innerView, rect, "", B_FOLLOW_LEFT_RIGHT, counter);
		diskviews.AddItem(dv);
		last = dv;
			
		counter++;
	}

	borderDiskInfo->ResizeTo(borderDiskInfo->Frame().IntegerWidth(),
		last->Frame().bottom+borderDiskInfo->Frame().IntegerHeight()-borderDiskInfo->innerView->Frame().IntegerHeight());
	ResizeTo(Frame().IntegerWidth(), borderDiskInfo->Frame().bottom + OFFSET);
}

void BInfoView :: DisplayDynamicDiskInfo()
{
	DiskView *dv;
	
	for(int i=0; i<diskviews.CountItems(); i++)
	{
		dv=(DiskView *)diskviews.ItemAt(i);
		dv->Update();
	}
}
